@@ -1,39 +1,59 @@
 #' Default options for CMF
-#' 
+#'
 #' A helper function that creates a list of options to be passed
 #' to \code{CMF}. To run the code with other option values, first
 #' run this function and then directly modify the entries before
 #' passing the list to \code{CMF}.
 #'
-#' Most of the parameters are for controlling the optimization, but some will alter
-#' the model itself. In particular, \code{useBias} is used for turning the bias terms
-#' on and off, and \code{method} will change the prior for \code{U}.
+#' Most of the parameters are for controlling the optimization, but some will
+#' alter the model itself. In particular, \code{useBias} is used for turning
+#' the bias terms on and off, and \code{method} will change the prior for
+#' \code{U}.
 #'
 #' The default choice for \code{method} is \code{"gCMF"}, providing the
-#' group-wise sparse CMF that identifies both shared and private factors (see Klami et al. (2013)
-#' for details). The value \code{"CMF"} turns off the group-wise sparsity, providing
-#' a CMF solution that attempts to learn only factors shared by all matrices.
-#' Finally, \code{method="GFA"} implements the group factor analysis (GFA) method, by
-#' fixing the variance of \code{U[[1]]} to one and forcing \code{useBias=FALSE}. Then \code{U[[1]]} can
-#' be interpreted as latent variables with unit variance and zero mean, as assumed by GFA
-#' and CCA (special case of GFA with M=2). Note that as a multi-view learning method \code{"GFA"}
-#' requires all matrices to share the same rows, the very first entity set.
+#' group-wise sparse CMF that identifies both shared and private factors
+#' (see Klami et al. (2013) for details). The value \code{"CMF"} turns off
+#' the group-wise sparsity, providing a CMF solution that attempts to learn
+#' only factors shared by all matrices. Finally, \code{method="GFA"} implements
+#' the group factor analysis (GFA) method, by fixing the variance of
+#' \code{U[[1]]} to one and forcing \code{useBias=FALSE}. Then \code{U[[1]]}
+#' can be interpreted as latent variables with unit variance and zero mean,
+#' as assumed by GFA and CCA (special case of GFA with M=2). Note that as a
+#' multi-view learning method \code{"GFA"} requires all matrices to share the
+#' same rows, the very first entity set.
 #'
 #' @return Returns a list of:
-#' \item{init.tau }{Initial value for the noise precisions. Only matters for Gaussian likelihood.}
-#' \item{init.alpha}{Initial value for the automatic relevance determination (ARD) prior precisions.}
-#' \item{grad.reg }{The regularization parameter for the under-relaxed Newton iterations. 0=no regularization, larger values provide inncreasing regularization. The value must be below 1.}
-#' \item{gradIter }{How many gradient steps for updating the projections are performed during each iteration of the whole algorithm. Default is one.}
-#' \item{grad.max }{Maximum absolute change for the elements of the projection matrices during one gradient step. Small values help to prevent over-shooting, wheres inf results to no constraints. Default is inf.}
+#' \item{init.tau}{Initial value for the noise precisions. Only matters for
+#'                 Gaussian likelihood.}
+#' \item{init.alpha}{Initial value for the automatic relevance determination
+#'                   (ARD) prior precisions.}
+#' \item{grad.reg }{The regularization parameter for the under-relaxed Newton
+#'                  iterations. 0 = no regularization, larger values provide
+#'                  increasing regularization. The value must be below 1.}
+#' \item{gradIter}{How many gradient steps for updating the projections are
+#'                  performed during each iteration of the whole algorithm.
+#'                  Default is 1.}
+#' \item{grad.max}{Maximum absolute change for the elements of the projection
+#'                 matrices during one gradient step. Small values help to
+#'                 prevent over-shooting, wheres inf results to no constraints.
+#'                 Default is \code{inf}.}
 #' \item{iter.max }{Number of iterations for the whole algorithm.}
-#' \item{computeCost }{Should the cost function values be computed or not. Defaults to TRUE.}
-#' \item{verbose }{0 = supress all printing, 1 = print current iteration and test RMSE every now and then, 2 = in addition to level 1 print also the current gradient norm.}
-#' \item{useBias }{Set this to FALSE to exclude the row and column bias terms. The default is TRUE.}
-#' \item{method }{Default value of "gCMF" computes the CMF with group-sparsity. The other possible values are "CMF" for turning off the group-sparsity prior, and "GFA" for implementing group factor analysis (and canonical correlation analysis when M=2).}
-#' \item{prior.alpha_0 }{Hyperprior values for the gamma prior for ARD.}
-#' \item{prior.alpha_0t }{Hyperprior values for the gamma prior for tau.}
+#' \item{computeCost}{Should the cost function values be computed or not.
+#'                    Defaults to \code{TRUE}.}
+#' \item{verbose}{0 = supress all printing, 1 = print current iteration and
+#'                test RMSE every now and then, 2 = in addition to level 1
+#'                print also the current gradient norm.}
+#' \item{useBias}{Set this to \code{FALSE} to exclude the row and column
+#'                bias terms. The default is \code{TRUE}.}
+#' \item{method}{Default value of "gCMF" computes the CMF with group-sparsity.
+#'               The other possible values are "CMF" for turning off the
+#'               group-sparsity prior, and "GFA" for implementing group factor
+#'               analysis (and canonical correlation analysis when
+#'               \code{M = 2}).}
+#' \item{prior.alpha_0}{Hyperprior values for the gamma prior for ARD.}
+#' \item{prior.alpha_0t}{Hyperprior values for the gamma prior for tau.}
 #' @author Arto Klami and Lauri Väre
-#' @seealso 'CMF' 
+#' @seealso 'CMF'
 #' @references
 #' Arto Klami, Guillaume Bouchard, and Abhishek Tripathi.
 #' Group-sparse embeddings in collective matrix factorization.
@@ -44,56 +64,64 @@
 #' International Conference on Artificial Intelligence and Statistics,
 #' volume 22 of JMLR:W&CP, pages 1269-1277, 2012.
 #' @examples
-#' 
-#' CMF_options = getCMFopts()
-#' CMF_options$iter.max = 500	#Change the number of iterations from default of 200 to 500.
-#' CMF_options$useBias = FALSE	#Don't take row and column means into consideration.
-#' #These options will be in effect when CMF_options is passed on to CMF.
 #'
+#' CMF_options = getCMFopts()
+#' CMF_options$iter.max = 500	# Change the number of iterations from default
+#'                            # of 200 to 500.
+#' CMF_options$useBias = FALSE # Do not take row and column means into
+#'                             # consideration.
+#' # These options will be in effect when CMF_options is passed on to CMF.
+#'
+#' @export
 getCMFopts <- function() {
-  # Initial value for the noise precisions (only matters for Gaussian likelihood)
+  # Initial value for the noise precisions (only matters for
+  # Gaussian likelihood)
   init.tau <- 10
-  
+
   # Initial value for the prior precisions
   init.alpha <- 5
-  
+
   # Parameters for the gradient optimization of the projection
   # matrices.
   # The algorithm is Newton-Raphson with diagonal Hessian
-  # and successive under-relaxation (=Richardson extrapolation),
-  # so that 
+  # and successive under-relaxation (= Richardson extrapolation),
+  # so that
   #  theta_new = grad.reg*theta_old + (1-grad.reg)(theta_old - g/h)
-  #  where g is gradient and h is Hessian. grad.reg should be between 0 and 1 to
-  #  stabilize the algorithm. 0 overshoots, 1 does not update anything.
+  #  where g is gradient and h is Hessian. grad.reg should be between 0 and 1
+  #  to stabilize the algorithm. 0 overshoots, 1 does not update anything.
   #
   # If grad.max < Inf, the step lengths are capped at grad.Max
   # grad.iter tells how many gradient steps to take within each update.
   grad.reg <- 0.7
   grad.iter <- 1
   grad.max <- Inf
-  
+
   # Parameters for controlling when the algorithm stops.
   iter.max <- 200
-  
+
   computeCost <- TRUE
   verbose <- 1 # 1=print progress every now and then, 0=supress all printing
-  useBias <- TRUE # Whether to include bias terms; useBias=FALSE means no bias terms
+  useBias <- TRUE # Whether to include bias terms;
+                  # useBias = FALSE means no bias terms
 
   # Whether to do GFA/CCA instead of CMF
-  # If this is set to "GFA" then useBias=FALSE and alpha is set to one for the first
-  # entity set, so that U[[1]] become latent variables with zero mean and unit variance.
-  # It this is set fo "CMF" the alpha-parameter becomes the same for all matrices.
+  # If this is set to "GFA" then useBias=FALSE and alpha is set to one for
+  # the first entity set, so that U[[1]] become latent variables with zero mean
+  # and unit variance.
+  # It this is set fo "CMF" the alpha-parameter becomes the same for all
+  # matrices.
   method <- "gCMF"
-  
+
   # Hyperparameters
   # - alpha_0, beta_0 for the ARD precisions
   # - alpha_0t, beta_0t for the residual noise predictions
   prior.alpha_0 <- prior.beta_0 <- 1
   prior.alpha_0t <- prior.beta_0t <- 0.001
-  
+
   return(list(init.tau=init.tau, init.alpha=init.alpha, grad.iter=grad.iter,
               iter.max=iter.max, verbose=verbose, computeCost=computeCost,
-              grad.reg=grad.reg, grad.max=grad.max, useBias=useBias, method=method,
+              grad.reg=grad.reg, grad.max=grad.max, useBias=useBias,
+              method=method,
               prior.alpha_0=prior.alpha_0, prior.beta_0=prior.beta_0,
               prior.alpha_0t=prior.alpha_0t, prior.beta_0t=prior.beta_0t))
 }
@@ -103,32 +131,38 @@ getCMFopts <- function() {
 #'
 #' Code for predicting missing elements with an existing
 #' CMF model. The predictions are made for all of the elements specified
-#' in the list of input matrices \code{X}. The function also returns the root mean square error (RMSE)
-#' between the predicted outputs and the values provided in \code{X}.
+#' in the list of input matrices \code{X}. The function also returns the
+#' root mean square error (RMSE) between the predicted outputs and the values
+#' provided in \code{X}.
 #'
 #' Note that \code{X} needs to be provided as a set
 #' of triplets instead of as a regular matrix. See
 #' \code{matrix_to_triplets}.
 #'
-#' @param X A list of sparse matrices specifying the indices for which to make the predictions.
-#' These matrices must correspond to the structure used for \code{X} when learning the
-#' model with \code{CMF}.
+#' @param X A list of sparse matrices specifying the indices for which to
+#'          make the predictions.
+#'          These matrices must correspond to the structure used for \code{X}
+#'          when learning the model with \code{CMF}.
 #' @param model A list of model parameter values provided by \code{CMF}.
 #' @return A list of
-#' \item{out }{A list of matrices corresponding to predictions for each matrix in \code{X}.}
-#' \item{error}{A vector containing the root-mean-square error for each matrix separately.}
+#' \item{out}{A list of matrices corresponding to predictions for each
+#'            matrix in \code{X}.}
+#' \item{error}{A vector containing the root-mean-square error for each
+#'              matrix separately.}
 #'
 #' @author Arto Klami and Lauri Väre
 #' @examples
-#' 
-#' #See CMF-package for an example.
 #'
+#' # See CMF-package for an example.
+#'
+#' @export
 predictCMF <- function(X,model) {
   D <- model$D
   inds <- model$inds
-  
+
   for(i in 1:length(X)){
-    if(!p_check_sparsity(X[[i]], model$D[model$inds[i,1]], model$D[model$inds[i,2]])) {
+    if(!p_check_sparsity(
+      X[[i]], model$D[model$inds[i,1]], model$D[model$inds[i,2]])) {
 	return(NULL)
     }
   }
@@ -137,18 +171,20 @@ predictCMF <- function(X,model) {
   for(m in 1:model$M) {
     out[[m]] <- X[[m]][,]
     # NOTE: Directly modifies out[[m]]
-    p_updatePseudoData(out[[m]],model$U[[model$inds[m,1]]],model$U[[model$inds[m,2]]],model$bias[[m]]$row$mu,model$bias[[m]]$col$mu)
+    p_updatePseudoData(
+      out[[m]],model$U[[model$inds[m,1]]],model$U[[model$inds[m,2]]],
+      model$bias[[m]]$row$mu,model$bias[[m]]$col$mu)
   }
   for(m in which(model$likelihood=="bernoulli")) {
     out[[m]][,3] <- exp(out[[m]][,3])
     out[[m]][,3] <- out[[m]][,3]/(1+out[[m]][,3])
-  } 
+  }
   for(m in which(model$likelihood=="poisson")) {
     out[[m]][,3] <- exp(out[[m]][,3])
     out[[m]][,3] <- log(1+out[[m]][,3])
   }
 
-  error <- rep(0,length(X))  
+  error <- rep(0,length(X))
   for(m in 1:length(X)) {
     for(r in 1:nrow(X[[m]])) {
       error[m] <- error[m] + (X[[m]][r,3]-out[[m]][r,3])^2
@@ -160,12 +196,12 @@ predictCMF <- function(X,model) {
 }
 
 #' Collective Matrix Factorization
-#' 
+#'
 #' Learns the CMF model for a given collection of M matrices.
 #' The code learns the parameters of a variational approximation
 #' for CMF, and also computes predictions for indices specified
 #' in \code{test}.
-#' 
+#'
 #' The variational approximation is fully factorized over all of
 #' the model parameters, including individual elements of the
 #' projection matrices. The parameters for the projection
@@ -183,49 +219,69 @@ predictCMF <- function(X,model) {
 #' For full description of the output parameters, see the
 #' referred publication. The notation in the code follows
 #' roughly the notation used in the paper.
-#' 
+#'
 #' @param X List of input matrices.
-#' @param inds A length(X) times 2 matrix that links dimensions of the matrices in X to object sets. inds[m, 1] tells which object set corresponds to the rows in matrix X[[m]], and inds[m, 2] tells the same for the columns.
+#' @param inds A \code{length(X)} times 2 matrix that links dimensions of the
+#'             matrices in \code{X} to object sets. \code{inds[m, 1]} tells
+#'             which object set corresponds to the rows in matrix
+#'             \code{X[[m]]}, and \code{inds[m, 2]} tells the same for the
+#'             columns.
 #' @param K The number of factors.
-#' @param opts A list of options as given by getCMFopts(). If set to NULL, the default values will be used.
-#' @param likelihood A list of likelihood choices, one for each matrix in X. Each entry should be a string with possible values of: "gaussian", "bernoulli" or "poisson".
+#' @param opts A list of options as given by \code{\link{getCMFopts}}.
+#'             If set to \code{NULL}, the default values will be used.
+#' @param likelihood A list of likelihood choices, one for each matrix in X.
+#'                   Each entry should be a string with possible values of:
+#'                   "gaussian", "bernoulli" or "poisson".
 #' @param D A vector containing sizes of each object set.
-#' @param test A list of test matrices. If not NULL, the code will
-#' compute predictions for these elements of the matrices. This duplicates the functionality of \code{predictCMF}.
+#' @param test A list of test matrices. If not NULL, the code will compute
+#'             predictions for these elements of the matrices. This duplicates
+#'             the functionality of \code{\link{predictCMF}}.
 #' @return A list of
-#' \item{U }{A list of the mean parameters for the rank-K projection matrices, one for each object set.}
-#' \item{covU }{A list of the variance parameters for the rank-K projection matrices, one for each object set.}
-#' \item{tau }{A vector of the precision parameter means.}
-#' \item{alpha }{A vector of the ARD parameter means.}
-#' \item{cost }{A vector of variational lower bound values.}
-#' \item{inds }{The input parameter \code{inds} stored for further use.}
-#' \item{errors }{A vector containing root-mean-square errors for each iteration, computed over the elements indicated by the \code{test} parameter.}
-#' \item{bias }{A list (of lists) storing the parameters of the row and column bias terms.}
-#' \item{D }{The sizes of the object sets as given in the parameters.}
-#' \item{K }{The number of components as given in the parameters.}
-#' \item{Uall }{Matrices of U joined into one sum(D) by K matrix, for easier plotting of the results.}
-#' \item{items }{A list containing the running number for each item among all object sets. This corresponds to rows of the Uall matrix. Each part of the list contains a vector that has the numbers for each particular object set.}
-#' \item{out }{If test matrices were provided, returns the reconstructed data sets. Otherwise returns NULL.}
-#' \item{M }{The number of input matrices.}
-#' \item{likelihood }{The likelihoods of the matrices.}
-#' \item{opts }{The options used for running the code.}
+#' \item{U}{A list of the mean parameters for the rank-K projection matrices,
+#'           one for each object set.}
+#' \item{covU}{A list of the variance parameters for the rank-K projection
+#'             matrices, one for each object set.}
+#' \item{tau}{A vector of the precision parameter means.}
+#' \item{alpha}{A vector of the ARD parameter means.}
+#' \item{cost}{A vector of variational lower bound values.}
+#' \item{inds}{The input parameter \code{inds} stored for further use.}
+#' \item{errors}{A vector containing root-mean-square errors for each
+#'               iteration, computed over the elements indicated by the
+#'               \code{test} parameter.}
+#' \item{bias}{A list (of lists) storing the parameters of the row and
+#'             column bias terms.}
+#' \item{D}{The sizes of the object sets as given in the parameters.}
+#' \item{K}{The number of components as given in the parameters.}
+#' \item{Uall}{Matrices of U joined into one sum(D) by K matrix, for
+#'             easier plotting of the results.}
+#' \item{items}{A list containing the running number for each item among
+#'              all object sets. This corresponds to rows of the \code{Uall}
+#'              matrix. Each part of the list contains a vector that has the
+#'              numbers for each particular object set.}
+#' \item{out}{If test matrices were provided, returns the reconstructed data
+#'            sets. Otherwise returns \code{NULL}.}
+#' \item{M}{The number of input matrices.}
+#' \item{likelihood}{The likelihoods of the matrices.}
+#' \item{opts}{The options used for running the code.}
 #' @author Arto Klami and Lauri Väre
 #' @references
 #' Arto Klami, Guillaume Bouchard, and Abhishek Tripathi.
 #' Group-sparse embeddings in collective matrix factorization.
 #' arXiv:1312.5921, 2013.
 #' @examples
-#' #See CMF-package for an example.
-#' 
-CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
+#' # See CMF-package for an example.
+#'
+#' @importFrom stats rnorm
+#' @export
+CMF <- function(X, inds, K, likelihood, D, test = NULL, opts = NULL) {
   if(is.null(opts)) {
     opts <- getCMFopts()
   }
 
   for(i in 1:length(X)){
-	if(!p_check_sparsity(X[[i]], D[inds[i,1]], D[inds[i,2]])){
-		return(NULL)
-	}
+  	if(!p_check_sparsity(X[[i]], D[inds[i,1]], D[inds[i,2]])){
+  		return(NULL)
+  	}
   }
 
   if(opts$method=="GFA") {
@@ -235,7 +291,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
       return(NULL)
     }
   }
-  
+
   # Construct the index sets and the symmetric matrix
   #
   # X are lists of observed values
@@ -252,22 +308,24 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
 
   N <- 0
   for(t in types) {
-    count <- D[t]  
+    count <- D[t]
     items[[t]] <- N+(1:count)
     N <- N + count
   }
- 
+
   alpha_0 <- opts$prior.alpha_0   # Easier access for hyperprior values
   beta_0 <- opts$prior.beta_0
   alpha_0t <- opts$prior.alpha_0t
   beta_0t <- opts$prior.beta_0t
 
   # ARD and noise parameters
-  alpha <- matrix(opts$init.alpha,C,K)          # The mean of the ARD precisions
-  b_ard <- matrix(0,C,K)          # The parameters of the Gamma distribution
-  a_ard <- alpha_0 + D/2          #     for ARD precisions
-  tau <- rep(opts$init.tau,M)     # The mean noise precisions
-  a_tau <- alpha_0t + rep(0,M)    # The parameters of the Gamma distribution
+  alpha <- matrix(opts$init.alpha, C, K)  # The mean of the ARD precisions
+  b_ard <- matrix(0, C, K)                # The parameters of the Gamma
+                                          #   distribution
+  a_ard <- alpha_0 + D / 2                # for ARD precisions
+  tau <- rep(opts$init.tau, M)            # The mean noise precisions
+  a_tau <- alpha_0t + rep(0, M)           # The parameters of the Gamma
+                                          #   distribution
   for(m in 1:M) {
      a_tau[m] <- a_tau[m] + nrow(X[[m]])/2
   }
@@ -276,7 +334,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
   # The projections
   U <- vector("list",length=C)
   covU <- vector("list",length=C) # The covariances
-  
+
   bias <- vector("list",length=M)
   for(i in 1:C) {
     # Random initialization
@@ -299,7 +357,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
       bias[[m]]$row$lambda <- 1
       bias[[m]]$row$scale <- 1
     }
-    
+
     bias[[m]]$col$mu <- rep(0,D[inds[m,2]])
     bias[[m]]$col$m <- 0
     if(!opts$useBias) {
@@ -322,17 +380,21 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
     # NOTE: Directly modifies xi[[m]]
     p_updatePseudoData(xi[[m]],U[[inds[m,1]]],U[[inds[m,2]]],
                                 bias[[m]]$row$mu,bias[[m]]$col$mu)
-    X[[m]][,3] <- xi[[m]][,3] - (1/(1+exp(-xi[[m]][,3])) - origX[[m]][,3])/tau[m]
-  } 
+    X[[m]][,3] <- xi[[m]][,3] -
+      (1 / (1 + exp(-xi[[m]][,3])) - origX[[m]][,3]) / tau[m]
+  }
 
   for(m in which(likelihood=="poisson")) {
     xi[[m]] <- X[[m]]
     maxval <- max(X[[m]][,3])
-    #print(paste("Maximal count in view ",m," is ",maxval,"; consider clipping if this is very large.",sep=""))
+    # print(paste("Maximal count in view ",m," is ",maxval,
+    #             "; consider clipping if this is very large.",sep=""))
     tau[m] <- 0.25+0.17*maxval
     p_updatePseudoData(xi[[m]],U[[inds[m,1]]],U[[inds[m,2]]],
                                 bias[[m]]$row$mu,bias[[m]]$col$mu)
-    X[[m]][,3] <- xi[[m]][,3] - (1/(1+exp(-xi[[m]][,3]))*(1-origX[[m]][,3]/log(1+exp(xi[[m]][,3]))))/tau[m]
+    X[[m]][,3] <- xi[[m]][,3] -
+      (1 / (1 + exp(-xi[[m]][,3])) *
+       (1 - origX[[m]][,3] / log(1 + exp(xi[[m]][,3])))) / tau[m]
   }
 
   cost <- vector()  # For storing the lower bounds
@@ -341,7 +403,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
   # The main loop
   #
   errors <- array(0,c(opts$iter.max,M))
-  for(iter in 1:opts$iter.max) {   
+  for(iter in 1:opts$iter.max) {
     #gc() # Just to make sure there are no memory leaks
     if((opts$verbose>0) & (iter%%10==1))
       print(paste("Iteration: ",iter,"/",opts$iter.max,sep=""))
@@ -367,7 +429,8 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
       par <- list(D=D,alpha=alpha,tau=tau,X=X,U=U,covU=covU,inds=inds,bias=bias)
       par$this <- i
 
-      # The inverse Hessian happens to be the covariance (see Ilin&Raiko, JMLR 2010)
+      # The inverse Hessian happens to be the covariance
+      # (see Ilin & Raiko, JMLR 2010)
       scale <- covU[[i]]
       reg <- opts$grad.reg
       for(n in 1:opts$grad.iter) {
@@ -375,7 +438,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
         if(max(abs(scale*g))>opts$grad.max) {
           scale <- opts$grad.max*scale/max(abs(scale*g))
         }
-        
+
         U[[i]] <- reg*U[[i]] + (1-reg)*(U[[i]] - scale*g)
         norm <- norm + sum(g^2)
 
@@ -383,25 +446,32 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
     }
     if((opts$verbose>1) & (iter%%10==1))
       print(paste("Gradient norm:",norm))
-    
+
     #
     # Update the mean profiles (and their priors)
     #
     if(opts$useBias) {
       for(m in 1:M) {
         # The approximations for each data point
-        temp <- p_updateMean(Xm = X[[m]],U[[inds[m,1]]],U[[inds[m,2]]],1,bias[[m]]$col$mu)
+        temp <- p_updateMean(
+          Xm = X[[m]],U[[inds[m,1]]],
+          U[[inds[m,2]]],
+          1,
+          bias[[m]]$col$mu)
         bias[[m]]$row$count <- temp$count
         bias[[m]]$row$nu <- 1/(tau[m]*bias[[m]]$row$count+1/bias[[m]]$row$scale)
-        bias[[m]]$row$mu <- (tau[m]*temp$sum + bias[[m]]$row$m/bias[[m]]$row$scale) *
+        bias[[m]]$row$mu <-
+          (tau[m] * temp$sum + bias[[m]]$row$m / bias[[m]]$row$scale) *
           bias[[m]]$row$nu
         # The approximation for the mean
         bias[[m]]$row$lambda <- 1/(1+1/bias[[m]]$row$scale*D[inds[m,1]])
-        bias[[m]]$row$m <- 1/bias[[m]]$row$scale*sum(bias[[m]]$row$mu)*bias[[m]]$row$lambda
-      
+        bias[[m]]$row$m <- 1 / bias[[m]]$row$scale *
+          sum(bias[[m]]$row$mu) * bias[[m]]$row$lambda
+
         # The scale
-        bias[[m]]$row$scale <- mean(bias[[m]]$row$nu + (bias[[m]]$row$mu - bias[[m]]$row$m)^2)
-      
+        bias[[m]]$row$scale <- mean(bias[[m]]$row$nu +
+          (bias[[m]]$row$mu - bias[[m]]$row$m) ^ 2)
+
         # The approximations for each data point
         temp <- p_updateMean(X[[m]],U[[inds[m,1]]],U[[inds[m,2]]],2,bias[[m]]$row$mu)
         bias[[m]]$col$count <- temp$count
@@ -411,13 +481,13 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
 
         # The approximation for the mean
         bias[[m]]$col$lambda <- 1/(1+1/bias[[m]]$col$scale*D[inds[m,2]])
-        bias[[m]]$col$m <- 1/bias[[m]]$col$scale*sum(bias[[m]]$col$mu)*bias[[m]]$col$lambda      
-      
+        bias[[m]]$col$m <- 1/bias[[m]]$col$scale*sum(bias[[m]]$col$mu)*bias[[m]]$col$lambda
+
         # The scale
         bias[[m]]$col$scale <- mean(bias[[m]]$col$nu + (bias[[m]]$col$mu - bias[[m]]$col$m)^2)
       }
     }
-    
+
     #
     # Update alpha, the ARD parameters
     #
@@ -438,7 +508,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
     if(opts$method=="GFA") {
       alpha[1,] <- 1
     }
-    
+
     #
     # Update tau, the noise precisions; only needed for Gaussian likelihood
     #
@@ -470,8 +540,8 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
                                   bias[[m]]$row$mu,bias[[m]]$col$mu)
       X[[m]][,3] <- xi[[m]][,3] - (1/(1+exp(-xi[[m]][,3]))*(1-origX[[m]][,3]/log(1+exp(xi[[m]][,3]))))/tau[m]
     }
-    
-    par <- list(D=D,alpha=alpha,tau=tau,X=X,U=U,covU=covU,inds=inds,this=1)  
+
+    par <- list(D=D,alpha=alpha,tau=tau,X=X,U=U,covU=covU,inds=inds,this=1)
 
     if(!is.null(test)) {
       out <- list()
@@ -484,7 +554,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
       for(m in which(likelihood=="bernoulli")) {
         out[[m]][,3] <- exp(out[[m]][,3])
         out[[m]][,3] <- out[[m]][,3]/(1+out[[m]][,3])
-      } 
+      }
       for(m in which(likelihood=="poisson")) {
         out[[m]][,3] <- exp(out[[m]][,3])
         out[[m]][,3] <- log(1+out[[m]][,3])
@@ -497,7 +567,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
       if((opts$verbose>0) & (iter%%10==1))
         print(paste("Test error:",paste(error,collapse=" ")))
     }
-    
+
     # Compute the cost function on training data
     if(opts$computeCost) {
       tcost <- 0
@@ -506,7 +576,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
           # The likelihood term
   	  logtau <- digamma(a_tau[m]) - log(b_tau[m])
           tcost <- tcost - dim(X[[m]])[1]*0.5*logtau + (b_tau[m]-a_tau[m])*tau[m]
-        
+
 	  # KL divergence for tau
 	  temp <- -lgamma(alpha_0t) + alpha_0t*log(beta_0t) + (alpha_0t-1)*logtau - beta_0t*tau[m] +
 	       lgamma(a_tau[m]) - a_tau[m]*log(b_tau[m]) - (a_tau[m]-1)*logtau + b_tau[m]*tau[m]
@@ -534,7 +604,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
             # The U and covU terms
 	    temp <- 0.5*dim(covU[[i]])[1]*logalpha - 0.5*sum(covU[[i]][,k] + U[[i]][,k]^2)*alpha[i,k] +
 	       0.5*sum(log(covU[[i]][,k]))
-	    tcost <- tcost - temp	
+	    tcost <- tcost - temp
 
             # The alpha part
 	    temp <- -lgamma(alpha_0) + alpha_0*log(beta_0) + (alpha_0-1)*logalpha - beta_0*alpha[i,k] +
@@ -552,27 +622,27 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
   	    temp <- 0.5*dim(covU[[i]])[1]*logalpha - 0.5*sum(covU[[i]][,k] + U[[i]][,k]^2)*alpha[i,k] +
 	       0.5*sum(log(covU[[i]][,k]))
 	    tcost <- tcost - temp
-	  }	
+	  }
 
           # The alpha part
 	  temp <- -lgamma(alpha_0) + alpha_0*log(beta_0) + (alpha_0-1)*logalpha - beta_0*alpha[i,k] +
 	     lgamma(sum(a_ard)) - sum(a_ard)*log(sum(b_ard[,k])) - (sum(a_ard)-1)*logalpha +
 	     sum(b_ard[,k])*alpha[i,k]
 	  tcost <- tcost - temp
-        }     
-      } 
+        }
+      }
       cost <- c(cost,tcost)
     }
-    
+
   } # the main loop of the algorithm ends
-  
+
   Uall <- U[[1]]
   if(C>1) {
     for(i in 2:C) {
       Uall <- rbind(Uall,U[[i]])
     }
   }
-  
+
   # Reconstructed datasets
   if(!is.null(test)) {
     out <- list()
@@ -585,7 +655,7 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
     for(m in which(likelihood=="bernoulli")) {
       out[[m]][,3] <- exp(out[[m]][,3])
       out[[m]][,3] <- out[[m]][,3]/(1+out[[m]][,3])
-    } 
+    }
     for(m in which(likelihood=="poisson")) {
       out[[m]][,3] <- exp(out[[m]][,3])
       out[[m]][,3] <- log(1+out[[m]][,3])
@@ -602,9 +672,9 @@ CMF <- function(X,inds,K,likelihood,D,test=NULL,opts=NULL) {
 
 
 
-#'
-#' Internal function for computing the gradients 
-#' 
+#
+# Internal function for computing the gradients
+#
 p_gradUsparseWrapper <- function(r,par,stochastic=FALSE) {
   g <- matrix(r,nrow=par$D[par$this])
   cur <- g
@@ -624,7 +694,7 @@ p_gradUsparseWrapper <- function(r,par,stochastic=FALSE) {
                            par$bias[[m]]$row$mu,par$bias[[m]]$col$mu)
     }
   }
-  
+
   for(m in which(par$inds[,2]==par$this)) {
     v1 <- par$inds[m,1]
     if(stochastic & m==1) {
@@ -636,14 +706,14 @@ p_gradUsparseWrapper <- function(r,par,stochastic=FALSE) {
                            par$bias[[m]]$row$mu,par$bias[[m]]$col$mu)
     }
   }
-  
+
   return(g)
 }
 
 
-#'
-#' Internal function for checking whether the input is in the right format
-#'
+#
+# Internal function for checking whether the input is in the right format
+#
 p_check_sparsity = function(mat, max_row, max_col){
 	if(class(mat) == "matrix"){	#if normal matrix
 		if(ncol(mat) != 3 | min(mat[,1:2]) < 1 | max(mat[,1]) > max_row | max(mat[,2]) > max_col){
@@ -658,7 +728,7 @@ p_check_sparsity = function(mat, max_row, max_col){
 }
 
 #' Conversion from matrix to coordinate/triplet format
-#' 
+#'
 #' The CMF code requires inputs to be speficied in a specific
 #' sparse format. This function converts regular R matrices
 #' into that format.
@@ -683,11 +753,12 @@ p_check_sparsity = function(mat, max_row, max_col){
 #' @author Arto Klami and Lauri Väre
 #' @seealso \code{triplets_to_matrix}
 #' @examples
-#' 
+#'
 #' x <- matrix(c(1,2,NA,NA,5,6),nrow=3)
 #' triplet <- matrix_to_triplets(x)
 #' print(triplet)
-#' 
+#'
+#' @export
 matrix_to_triplets = function(orig) {
   triplets = matrix(0, nrow = length(which(!is.na(orig))), ncol = 3)
   count = 1
@@ -703,7 +774,7 @@ matrix_to_triplets = function(orig) {
 }
 
 #' Conversion from triplet/coordinate format to matrix
-#' 
+#'
 #' This function is the inverse of \code{matrix_to_triplets}.
 #' It converts a matrix represented as a set of triplets into
 #' an object of the class \code{matrix}. The missing entries
@@ -712,18 +783,19 @@ matrix_to_triplets = function(orig) {
 #'
 #' See \code{matrix_to_triplets} for a description of the
 #' representation.
-#' 
+#'
 #' @param triplets A matrix in triplet/coordinate format
 #' @return The input matrix as a normal matrix of class \code{matrix}
 #' @author Arto Klami and Lauri Väre
 #' @examples
-#' 
+#'
 #' x <- matrix(c(1,2,NA,NA,5,6),nrow=3)
 #' triplet <- matrix_to_triplets(x)
 #' print(triplet)
 #' xnew <- triplets_to_matrix(triplet)
 #' print(xnew)
-#' 
+#'
+#' @export
 triplets_to_matrix = function(triplets) {
   mat = matrix(NA, nrow = max(triplets[,1]), ncol = max(triplets[,2]))
   for(t in 1:nrow(triplets)){
